/*
 *  Copyright 2003-2014  Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.games1729.shariks;

import java.awt.*;

class DefaultSharikLook implements SharikLook {
    private transient Image[] scaledHeadImage = new Image[8];
    private transient Image scaledTailImage = null;
    private transient Image scaledCrazyImage = null;
    
    private PlayerColorFilter colorFilter;

    DefaultSharikLook() {
	this(Color.GRAY);
    }

    DefaultSharikLook(Color color) {
	this.colorFilter = new PlayerColorFilter(color);
    }

    private Image scale(Image image, Image cached, float r, Field field) {
	return field.game.renderer.scale(image, cached, 
					 (int)(2 * r * field.getScale()), 
					 (int)(2 * r * field.getScale()),
					 colorFilter);
    }

    private Image getScaledPlayerImage(int orientation, Field field) {
	scaledHeadImage[orientation] = 
	    scale(field.game.skin.getHeadImage(orientation),
		  scaledHeadImage[orientation], Player.RADIUS, field);
	return scaledHeadImage[orientation];
    }

    private Image getScaledTailImage(Field field) {
	scaledTailImage = scale(field.game.skin.getTailImage(),
				scaledTailImage, TailSharik.RADIUS, field);
	return scaledTailImage;
    }

    private Image getScaledCrazyImage(Field field) {
	scaledCrazyImage = scale(field.game.skin.getCrazyImage(),
				 scaledCrazyImage, CrazySharik.RADIUS, field);
	return scaledCrazyImage;
    }

    public void render(Sharik s, Graphics2D g, int layer) {
	Image image;
	if (s instanceof Player)
	    image = getScaledPlayerImage(((Player)s).getOrientation(), s.field);
	else if (s instanceof TailSharik)
	    image = getScaledTailImage(s.field);
	else
	    image = getScaledCrazyImage(s.field);

	g.drawImage(image, -image.getWidth(null) / 2, -image.getHeight(null) / 2, null);
    }

    private void readObject(java.io.ObjectInputStream in)
	throws java.io.IOException, ClassNotFoundException {	
	
	in.defaultReadObject();
	scaledHeadImage = new Image[8];
    }
}
