/*
 *  Copyright 2003-2014  Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.games1729.shariks;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * Human player.
 */
class Keyboarder extends Player implements KeyListener {
    public static final int KEYS_1 = 0;
    public static final int KEYS_2 = 1;
    public static final int KEYS_3 = 2;
    public static final int KEYS_4 = 3;

    private transient int direction = -1;
    private int keys;

    Keyboarder(Field field, Color color, String name, int keys) {
	super(field, color, name);
	
	this.keys = keys;
    }

    void setField(Field field) {
	if (this.field != null)
	    this.field.game.component.removeKeyListener(this);
	super.setField(field);
	if (this.field != null)
	    this.field.game.component.addKeyListener(this);
    }

    void start() {
	field.game.component.addKeyListener(this);
    }

    void stop() {
	field.game.component.removeKeyListener(this);
    }

    public void keyPressed(KeyEvent e) {
	int d = keyCodeToDirection(e.getKeyCode());
	if (d < 0) return;

	if (direction < 0)
	    direction = d;
	else if ((d + 2) % 8 == direction)
	    direction = (d + 1) % 8;
	else if ((direction + 2) % 8 == d)
	    direction = (direction + 1) % 8;
	else
	    direction = d;

	//System.err.println(direction);
    }
    
    public void keyReleased(KeyEvent e) {
	if (direction < 0) return;	   

	int d = keyCodeToDirection(e.getKeyCode());
	if (d < 0) return;
	
	if (direction == d)
	    direction = -1;
	else if ((d + 1) % 8 == direction)
	    direction = (direction + 1) % 8;
	else if ((direction + 1) % 8 == d)
	    direction = (direction + 7) % 8;

        //System.err.println(direction);
    }

    private int keyCodeToDirection(int keyCode) {
        //System.err.println(keyCode);

	switch (keys) {
	case KEYS_1:
	    switch (keyCode) {
	    case 38:
		return NORTH;
	    case 39:
		return EAST;
	    case 40:
		return SOUTH;
	    case 37:
		return WEST;
	    }
	    break;
	case KEYS_2:
	    switch (keyCode) {
	    case 80:
		return NORTH;
	    case 222:
		return EAST;
	    case 59:
		return SOUTH;
	    case 76:
		return WEST;
	    }
	    break;
	case KEYS_3:
	    switch (keyCode) {
	    case 89:
		return NORTH;
	    case 74:
		return EAST;
	    case 72:
		return SOUTH;
	    case 71:
		return WEST;
	    }
	    break;
	case KEYS_4:
	    switch (keyCode) {
	    case 87:
		return NORTH;
	    case 68:
		return EAST;
	    case 83:
		return SOUTH;
	    case 65:
		return WEST;
	    }
	    break;
	}

	return -1;
    }

    protected int getDirection() {
	return direction;
    }

    public void keyTyped(KeyEvent e) {}

    private void readObject(java.io.ObjectInputStream in)
	throws java.io.IOException, ClassNotFoundException {	
	
	in.defaultReadObject();
	direction = -1;
    }
}
